#include "tools.h"
void CpcahemvU(N, alpha, A, IA, JA, descA, X0, IX0, JX0, descX0, incX0, beta, 
               Y0, IY0, JY0, descY0, incY0)
int N;
float alpha;
float *A;
int IA;
int JA;
int *descA;
float *X0;
int IX0;
int JX0;
int *descX0;
int incX0;
float beta;
float *Y0;
int IY0;
int JY0;
int *descY0;
int  incY0;
/*
 *
 *             ======(Yr)======
 *             ======(Xr)======
 *                    N
 *              ---------------
 *  |    |     |\_             |          A - N x N
 *  |    |     |  \_           |          Y - N x 1
 *  |    |     |    \_   (A)   |          X - 1 x N
 * (Yc) (Xr) N |      \_       |
 *  |    |     |        \_     |          X will be replicated on every
 *  |    |     |          \_   |            process row and column
 *  |    |     |            \  |          Y will have space on every
 *  |    |     |             \_|            process row and column
 *              ---------------
 */
{
/*
 * .. External routines ..
 */
   char *ptop();
   void pchkmat();
   void pchkvec();
   void pberror_();
   void Cinfog2l();
   int Cnumroc2();
   void Cblacs_gridinfo();
   void Csgebs2d();
   void Csgebr2d();
   void Csgsum2d();
   F_INTG_FCT sgemv_();
   void Cpcsvabs1();
   void Cpsscal1();
   void Cpscopy1();
   void Cpsaxpy1();

   F_CHAR trans;
   char *top;
   int ctxt, nprow, npcol, myrow, mycol;
   int Ir, Ic, Jr, Jc, inc;
   int descXr[DLEN_], descXc[DLEN_], descYr[DLEN_], descYc[DLEN_];
   int i, j, k, kb, nb, lld, LOCp, LOCq, LOCp2, LOCq2, info=0, one=1, I=0;
   int arow, acol, currow, curcol;
   float *Xr, *Xc, *Yr, *Yc, *xr, *xc, *yr, *yc, *yr2, *yc2;
   float *absA, *a, *ar, *ac, *aa;
   float zero=0;
   int IX1, JX1, incX1, descX1[DLEN_];
   float *X1;

   ctxt = descA[CTXT_];
   Cblacs_gridinfo(descA[CTXT_], &nprow, &npcol, &myrow, &mycol);
/*
 * Scale Y0 by beta: Y0 = beta * Y0; this allows us to later add in
 * alpha*A*x to get Y0 = alpha*A*x + Y0*beta
 */
   Cpsscal1(N, beta, Y0, IY0, JY0, descY0, incY0);
/*
 * Collapse complex X down to real X1
 */
   Cpcsvabs1(N, X0, IX0, JX0, descX0, incX0, &X1, &IX1, &JX1, descX1, &incX1);
/*
 * Get local information about our matrix
 */
   Cinfog2l(IA, JA, descA, nprow, npcol, myrow, mycol, &i, &j, &arow, &acol);
   a = &A[ 2*(i+j*descA[LLD_]) ];
   nb = descA[NB_];
   lld = descA[LLD_];
   LOCp2 = LOCp = Cnumroc2(N, IA, nb, myrow, descA[RSRC_], nprow);
   LOCq2 = LOCq = Cnumroc2(N, JA, nb, mycol, descA[CSRC_], npcol);

/*
 * Set up Xr and Xc, and copy abs( X ) to one row/col.
 */
   inc = 1;
   Ir = Jc = 0;
   Jr = JA % nb;
   Ic = IA % nb;
   i = Jr + LOCq;
   j = Ic + LOCp;
   Mmalloc(Xr, float, i+j, info, ctxt);
   Xc = &Xr[i];
   Mdescset(descXr, 1, N + Jr, nb, nb, descX1[RSRC_], acol, ctxt, 1);
   Mdescset(descXc, N + Ic, 1, nb, nb, arow, descX1[CSRC_], ctxt, MAX(j,1));
   Cpscopy1(N, X1, IX1, JX1, descX1, inc, Xr, Ir, Jr, descXr, inc);
   Cpscopy1(N, X1, IX1, JX1, descX1, inc, Xc, Ic, Jc, descXc, inc);
   free(X1);
/*
 * Set up Yr and Yc
 */
   Mmalloc(Yr, float, i+j, info, ctxt);
   Yc = &Yr[i];
   Mdescset(descYc, N + Ic, 1, nb, nb, arow,
            MCindxg2p(JY0, descY0[NB_], descY0[CSRC_], npcol), ctxt, MAX(j,1));
   Mdescset(descYr, 1, N + Jr, nb, nb,
            MCindxg2p(IY0, descY0[MB_], descY0[RSRC_], nprow), acol, ctxt, 1);
/*
 * Get memory for absA
 */
   Mmalloc(absA, float, nb*MAX(LOCp, LOCq), info, ctxt);

/*
 * Set local pointers
 */
   if (myrow == arow)
   {
      xc = &Xc[Ic];
      yc2 = yc = &Yc[Ic];
   }
   else
   {
      xc = Xc;
      yc2 = yc = Yc;
   }
   if (mycol == acol)
   {
      xr = &Xr[Jr];
      yr2 = yr = &Yr[Jr];
   }
   else
   {
      xr = Xr;
      yr2 = yr = Yr;
   }
   ac = ar = a;

/*
 * Broadcast X so all rows and columns have a copy
 */
   top = ptop("B", "C", "!");
   if (myrow == descXr[RSRC_]) Csgebs2d(ctxt, "c", top, LOCq, 1, xr, LOCq);
   else Csgebr2d(ctxt, "c", top, LOCq, 1, xr, LOCq, descXr[RSRC_], mycol);
   top = ptop("B", "R", "!");
   if (mycol == descXc[CSRC_]) Csgebs2d(ctxt, "r", top, LOCp, 1, xc, LOCp);
   else Csgebr2d(ctxt, "r", top, LOCp, 1, xc, LOCp, myrow, descXc[CSRC_]);
/*
 * Loop over global matrix
 */
   kb = nb - Ic;
   kb = MIN(kb, N);
   currow = arow;
   curcol = acol;
   LOCp = 0;
   do
   {
      if (mycol == curcol)
      {
         trans = C2F_CHAR("T");
         aa = absA;
         a = ac;
         if (myrow == currow)  /* I have diagonal blocks */
         {
/*
 *          Do block along column
 */
            for (j=0; j != kb; j++)
            {
               k = LOCp + j;
               for (i=0; i != k; i++) aa[i] = ABS( a[2*i] ) + ABS( a[2*i+1] );
               for (k=LOCp+kb; i != k; i++) aa[i] = 0.0;
               aa += k;
               a += 2*lld;
            }
            LOCp += kb;
            sgemv_(trans, &LOCp, &kb, &alpha, absA, &LOCp, xc, &one, &zero,
                   yr, &one);
/*
 *          Do block along row
 */
            trans = C2F_CHAR("N");
            aa = absA;
            a = ar;
            for (j=0; j != kb; j++)  /* copy diagonal block */
            {
               for (i=0; i <= j; i++) aa[i] = ABS( a[2*i] ) + ABS( a[2*i+1] );
               for (; i != kb; i++) aa[i] = 0.0;
               aa += kb;
               a += 2*lld;
            }
            for (j=LOCq-kb; j; j--)
            {
               for (i=0; i != kb; i++) aa[i] = ABS( a[2*i] ) + ABS( a[2*i+1] );
               aa += kb;
               a += 2*lld;
            }
            sgemv_(trans, &kb, &LOCq, &alpha, absA, &kb, xr, &one, &zero,
                   yc, &one);
            ar += 2*kb*(lld + 1);
            yc += kb;
         }
         else
         {
            ar += 2*kb*lld;
            if (LOCp)
            {
               for (j=kb; j; j--)
               {
                  for (i=0; i != LOCp; i++)
                     aa[i] = ABS( a[2*i] ) + ABS( a[2*i+1] );
                  aa += LOCp;
                  a += 2*lld;
               }
               sgemv_(trans, &LOCp, &kb, &alpha, absA, &LOCp, xc, &one, &zero,
                      yr, &one);
            }
            else for (i=0; i != kb; i++) yr[i] = 0.0;
         }
         LOCq -= kb;
         xr += kb;
         yr += kb;
         ac += 2*kb*lld;
      }
      else if (myrow == currow)
      {
         LOCp += kb;
         if (LOCq)
         {
            trans = C2F_CHAR("N");
            a = ar;
            aa = absA;
            for (j=LOCq; j; j--)
            {
               for (i=0; i != kb; i++) aa[i] = ABS( a[2*i] ) + ABS( a[2*i+1] );
               aa += kb;
               a += 2*lld;
            }
            sgemv_(trans, &kb, &LOCq, &alpha, absA, &kb, xr, &one, &zero,
                   yc, &one);
         }
         else for (i=0; i != kb; i++) yc[i] = 0.0;
         ar += 2*kb;
         yc += kb;
      }
      I += kb;
      kb = MIN(nb, N-I);
      if (++currow == nprow) currow=0;
      if (++curcol == npcol) curcol=0;
   }
   while (I != N);
   free(Xr);
   free(absA);
/*
 * Collect distributed Yr and Yc
 */
   if (LOCq2 != 0)
   {
      top = ptop("C", "C", "!");
      Csgsum2d(ctxt, "col", top, LOCq2, 1, yr2, LOCq2, descYr[RSRC_], mycol);
   }
   if (LOCp2 != 0)
   {
      top = ptop("C", "R", "!");
      Csgsum2d(ctxt, "row", top, LOCp2, 1, yc2, LOCp2, myrow, descYr[CSRC_]);
   }
/*
 * Set y = yc + yr
 */
   if (incY0 == descY0[M_])
   {
      Cpsaxpy1(N, 1.0, Yc, Ic, Jc, descYc, inc, Yr, Ir, Jr, descYr, inc);
      Cpsaxpy1(N, 1.0, Yr, Ir, Jr, descYr, inc, Y0, IY0, JY0, descY0, incY0);
   }
   else
   {
      Cpsaxpy1(N, 1.0, Yr, Ir, Jr, descYr, inc, Yc, Ic, Jc, descYc, inc);
      Cpsaxpy1(N, 1.0, Yc, Ic, Jc, descYc, inc, Y0, IY0, JY0, descY0, incY0);
   }
   free(Yr);
}
